#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.simpleextractor import SimpleExtractor
from ykdl.util.html import get_content, add_header
from ykdl.util.match import match1, matchall

class ZYLive(SimpleExtractor):
    name = u"ZhangYu Live (章鱼直播)"

    def __init__(self):
        SimpleExtractor.__init__(self)
        add_header('User-Agent', 'Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3')
        self.live = True
        self.title_pattern = '<title>([^<]+)'
        self.url_pattern = '<video _src=\'([^\']+)'
        self.artist_pattern = 'videoTitle = \"([^\"]+)'

site = ZYLive()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.extractor import VideoExtractor
from ykdl.videoinfo import VideoInfo
from ykdl.util.html import get_content
from ykdl.util.match import match1, matchall
from ykdl.compact import compact_bytes

import hashlib
import json

SECKEY = "RTYDFkHAL$#%^tsf_)(*^Gd%$"

class Chushou(VideoExtractor):
    name = u'Chushou Live (触手直播)'

    def prepare(self):
        info = VideoInfo(self.name, True)
        self.vid = match1(self.url, '(\d+).htm')
        api_url = "https://chushou.tv/live-room/get-play-url.htm"
        t = get_content("https://chushou.tv/timestamp/get.htm")
        sign_context = "_t={}&protocols=1,2&roomId={}".format(t,self.vid)
        sign = hashlib.md5(compact_bytes(SECKEY+'&'+sign_context, 'utf-8')).hexdigest()

        data = json.loads(get_content(api_url+'?'+sign_context+'&_sign={}'.format(sign)))

        assert data['code'] == 0, data['message']

        info.stream_types.append("current")
        info.streams["current"] = {'container': 'flv', 'video_profile': 'current', 'src' : [data["data"][0]['shdPlayUrl']], 'size': float('inf')}

        return info

site = Chushou()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from ykdl.util.html import get_location

def get_extractor(url):
    if not 'bangumi' in url:
        url = get_location(url)
    if re.search("live.bili", url):
        from . import live as s
    elif re.search("vc.bili", url):
        from . import vc as s
    elif re.search("bangumi", url):
        from . import bangumi as s
    else:
        from . import video as s
    return s.site



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.extractor import VideoExtractor
from ykdl.videoinfo import VideoInfo
from ykdl.util.html import get_content, add_header
from ykdl.util.match import match1, matchall

class HuyaLive(VideoExtractor):
    name = u"Huya Live (虎牙直播)"

    def prepare(self):
        info = VideoInfo(self.name)

        html  = get_content(self.url)

        channel = match1(html, '"channel":"*(\d+)"*,')
        sid = match1(html, '"sid":"*(\d+)"*,')
        assert not channel == '0' and not sid == '0', "live video is offline"


        _hex = '0000009E10032C3C4C56066C6976657569660D6765744C6976696E67496E666F7D0000750800010604745265711D0000680A0A0300000000000000001620FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF2600361777656226323031377633322E313131302E33266875796146005C0B1300000000{}2300000000{}3300000000000000000B8C980CA80C'
        hex_content = _hex.format( format(int(channel), 'X').zfill(8), format(int(sid), 'X').zfill(8) )
        video_data = get_content('http://cdn.wup.huya.com/', data = bytearray.fromhex(hex_content), charset = 'ignore').decode("utf-8",  errors='ignore')
        assert channel in video_data, "live video is offline"

        vid = match1(video_data, '(%s-%s[^f]+)'%(channel, sid))

        wsSecret = match1(video_data, 'wsSecret=([0-9a-z]{32})')
        wsTime = match1(video_data , 'wsTime=([0-9a-z]{8})')
        line = match1(video_data, '://(.+\.(flv|stream)\.huya\.com/(hqlive|huyalive))')
        flv_url = 'http://{}/{}.flv?wsSecret={}&wsTime={}'.format(line, vid, wsSecret, wsTime)
        info.stream_types.append("current")
        info.streams["current"] = {'container': 'mp4', 'video_profile': "current", 'src': [flv_url], 'size' : float('inf')}
        return info

site = HuyaLive()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.extractor import VideoExtractor
from ykdl.videoinfo import VideoInfo
from ykdl.util.html import get_content, add_header, fake_headers, get_location
from ykdl.util.match import match1, matchall
from xml.dom.minidom import parseString


def parse_cid_playurl(xml):
    urls = []
    size = 0
    doc = parseString(xml.encode('utf-8'))
    ext = doc.getElementsByTagName('format')[0].firstChild.nodeValue
    qlt = doc.getElementsByTagName('quality')[0].firstChild.nodeValue
    for durl in doc.getElementsByTagName('durl'):
        urls.append(durl.getElementsByTagName('url')[0].firstChild.nodeValue)
        size += int(durl.getElementsByTagName('size')[0].firstChild.nodeValue)
    return urls, size, ext, qlt

class BiliBase(VideoExtractor):
    supported_stream_profile = [u'蓝光', u'超清', u'高清', u'流畅']
    profile_2_type = {u'蓝光': 'BD', u'超清': 'TD', u'高清': 'HD', u'流畅' :'SD'}
    qlt_2_profile = {'80': u'蓝光', '64': u'超清',  '32': u'高清', '16': u'流畅'}

    def prepare(self):
        info = VideoInfo(self.name)
        add_header("Referer", "http://www.bilibili.com")
        info.extra["referer"] = "http://www.bilibili.com"
        info.extra["ua"] = fake_headers['User-Agent']

        self.vid, info.title = self.get_vid_title()

        assert self.vid, "can't play this video: {}".format(self.url)

        for q in self.supported_stream_profile:
            api_url = self.get_api_url(self.supported_stream_profile.index(q))
            html = get_content(api_url)
            self.logger.debug("HTML> {}".format(html))
            code = match1(html, '<code>([^<])')
            if code:
                continue
            urls, size, fmt, qlt = parse_cid_playurl(html)
            if 'mp4' in fmt:
                ext = 'mp4'
            elif 'flv' in fmt:
                ext = 'flv'

            prf = self.qlt_2_profile[qlt]
            st = self.profile_2_type[prf]
            if st in info.stream_types:
               continue
            info.stream_types.append(st)
            info.streams[st] = {'container': ext, 'video_profile': prf, 'src' : urls, 'size': size}
        assert len(info.stream_types), "can't play this video!!"
        return info



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.html import get_location, get_content
from ykdl.util.match import match1
from ykdl.compact import compact_bytes, urlencode

import json
import time
import hashlib

from .bilibase import BiliBase

SEC2 = '9b288147e5474dd2aa67085f716c560d'
bangumi_api_url = 'http://bangumi.bilibili.com/player/web_api/playurl?'

class BiliBan(BiliBase):
    name = u'哔哩哔哩 番剧 (Bilibili Bangumi)'

    def get_vid_title(self):
        if not 'bangumi' in self.url:
            self.url = get_location(self.url)

        html = get_content(self.url)
        title = match1(html, '<h1 title="([^"]+)', '<title>([^<]+)').strip()

        vid = match1(html, '\"cid\":(\d+)')

        return vid, title

    def get_api_url(self, q):
        if "movie" in self.url:
            mod = "movie"
        else:
            mod = "bangumi"
        ts = str(int(time.time()))
        params_str = 'cid={}&module={}&player=1&quality={}&ts={}'.format(self.vid, mod, q, ts)
        chksum = hashlib.md5(compact_bytes(params_str + SEC2, 'utf8')).hexdigest()
        return bangumi_api_url + params_str + '&sign=' + chksum

    def prepare_list(self):
        html = get_content(self.url)
        sid = match1(html, 'var season_id = "(\d+)";')
        j_ = get_content("https://bangumi.bilibili.com/jsonp/seasoninfo/{}.ver".format(sid))[19:-2]
        s_data = json.loads(j_)
        urls = [e['webplay_url'] for e in sorted(s_data['result']['episodes'], key=lambda e: e['index'])]
        return urls

site = BiliBan()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ykdl.util.html import get_content
from ykdl.util.match import match1, matchall
from ykdl.compact import compact_bytes

import hashlib
import json
import time

from .bilibase import BiliBase

appkey='f3bb208b3d081dc8'
SECRETKEY_MINILOADER = '1c15888dc316e05a15fdd0a02ed6584f'

class BiliVideo(BiliBase):
    name = u'哔哩哔哩 (Bilibili)'

    def get_vid_title(self):
        if "#page=" in self.url:
            page_index = match1(self.url, '#page=(\d+)')
            av_id = match1(self.url, '\/(av\d+)')
            self.url = 'http://www.bilibili.com/{}/index_{}.html'.format(av_id, page_index)
        if not self.vid:
            html = get_content(self.url)
            vid = match1(html, 'cid=(\d+)', 'cid=\"(\d+)', '\"cid\":(\d+)')
            title = match1(html, '<h1 title="([^"]+)', '<title>([^<]+)').strip()

        return vid, title

    def get_api_url(self, q):
        t = int(time.time())
        sign_this = hashlib.md5(compact_bytes('cid={}&player=1&quality={}&ts={}{}'.format(self.vid, q, t, SECRETKEY_MINILOADER), 'utf-8')).hexdigest()
        return 'http://interface.bilibili.com/playurl?cid={}&player=1&quality={}&ts={}&sign={}'.format(self.vid, q, t, sign_this)

    def prepare_list(self):
        html = get_content(self.url)
        video_list = matchall(html, ['<option value=\'([^\']*)\''])
        if video_list:
            return ['http://www.bilibili.com'+v for v in video_list]

site = BiliVideo()
